package flink.zanxiangnet.ad.monitoring.trigger;

import flink.zanxiangnet.ad.monitoring.pojo.entity.AdDataOfMinuteODS;
import flink.zanxiangnet.ad.monitoring.util.DateUtil;
import org.apache.flink.streaming.api.windowing.triggers.Trigger;
import org.apache.flink.streaming.api.windowing.triggers.TriggerResult;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;

import java.time.LocalDateTime;

public class AdMinuteODSStreamTrigger extends Trigger<AdDataOfMinuteODS, TimeWindow> {
    /**
     * @param adDataOfMinuteODS
     * @param time              此处指 eventTime
     * @param timeWindow        start：窗口开始时间，end：窗口关闭时间，maxTimestamp：能接收到数据的的最大时间
     * @param triggerContext    currentWatermark：当前水印时间，第一个元素进入的时候为 Long.MAX
     * @return
     * @throws Exception
     */
    @Override
    public TriggerResult onElement(AdDataOfMinuteODS adDataOfMinuteODS, long time, TimeWindow timeWindow, TriggerContext triggerContext) throws Exception {
        LocalDateTime beginDateTime = DateUtil.milliToLocalDateTime(timeWindow.getStart());
        if (adDataOfMinuteODS.getHour() == beginDateTime.getHour()) {
            return TriggerResult.FIRE;
        }
        // 回滚上小时的数据
        if (beginDateTime.getHour() > 0 && (adDataOfMinuteODS.getHour() == beginDateTime.getHour() - 1)) {
            int minute = beginDateTime.getMinute();
            // 分别在 5分钟、20分钟、1小时后重新计算上小时的数据
            if (minute == 0 || minute == 15 || minute == 55) {
                return TriggerResult.FIRE;
            }
        }
        return TriggerResult.CONTINUE;
    }

    @Override
    public TriggerResult onProcessingTime(long time, TimeWindow timeWindow, TriggerContext triggerContext) throws Exception {
        return TriggerResult.CONTINUE;
    }

    /**
     * @param time           eventTime
     * @param timeWindow
     * @param triggerContext
     * @return
     * @throws Exception
     */
    @Override
    public TriggerResult onEventTime(long time, TimeWindow timeWindow, TriggerContext triggerContext) throws Exception {
        return TriggerResult.FIRE_AND_PURGE;
    }

    @Override
    public void clear(TimeWindow timeWindow, TriggerContext triggerContext) throws Exception {
    }
}
